use crate::state::ReqState;
use axum::{
    body::Empty,
    extract::{Path, Query, State},
    response::IntoResponse,
};
use hyper::StatusCode;
use serde::Deserialize;

#[derive(Deserialize)]
pub struct UploadQuery {
    revocation_key: String,
}

pub async fn delete(
    State(state): ReqState,
    Query(query): Query<UploadQuery>,
    Path(id): Path<String>,
) -> impl IntoResponse {
    if id.len() < 8 {
        return Err(StatusCode::NOT_FOUND);
    }

    let id = id[..8].to_string();

    let row = sqlx::query!("select revocation_key from files where id = $1", id)
        .fetch_one(&state.db)
        .await
        .map_err(|_| StatusCode::NOT_FOUND)?;

    if row.revocation_key != query.revocation_key {
        return Err(StatusCode::UNAUTHORIZED);
    }

    let path = state.files_dir.join(id.clone());
    if path.exists() {
        tokio::fs::remove_file(path)
            .await
            .map_err(|_| StatusCode::INTERNAL_SERVER_ERROR)?;
    }

    sqlx::query!("delete from files where id = $1", id)
        .execute(&state.db)
        .await
        .map_err(|_| StatusCode::INTERNAL_SERVER_ERROR)?;

    Ok((StatusCode::NO_CONTENT, Empty::new()))
}
