use crate::state::ReqState;
use axum::{
    extract::{Path, State},
    response::IntoResponse,
};
use hyper::{Body, Request, StatusCode, Uri};
use std::{path::PathBuf, str::FromStr};
use tower::util::ServiceExt;
use tower_http::services::ServeFile;

pub async fn get(State(state): ReqState, Path(id): Path<String>) -> impl IntoResponse {
    if id.len() < 8 {
        return Err(StatusCode::NOT_FOUND);
    }

    let extension = PathBuf::from_str(&id).map_err(|_| StatusCode::NOT_FOUND)?;
    let extension = extension.extension().and_then(|ext| ext.to_str());

    let id = id[..8].to_string();
    let path = state.files_dir.join(id.clone());

    if path.exists() {
        let req = Request::builder()
            .uri(Uri::from_static("/"))
            .body(Body::empty())
            .unwrap();

        if let Some(ext) = extension {
            let mime = mime_guess::from_ext(ext).first_or_octet_stream();
            let serve_file = ServeFile::new_with_mime(path, &mime).oneshot(req);
            Ok(serve_file.await.unwrap().map(axum::body::boxed))
        } else {
            let serve_file = ServeFile::new(path).oneshot(req);
            Ok(serve_file.await.unwrap().map(axum::body::boxed))
        }
    } else {
        Err(StatusCode::NOT_FOUND)
    }
}
